from .event_listener import EventListener
from .utility import Utility
from .stoppable_thread import StoppableThread
from .signal_handler import SignalHandler

__all__ = ["EventListener", "Utility", "StoppableThread", "SignalHandler"]



from enum import Enum
from dataclasses import dataclass
from typing import Optional


class DownloadEventType(str, Enum):
    COMPLETED = "completed"
    """The download has been completed"""

    FAILED = "failed"
    """The download has been failed"""

    PROGRESS = "progress"
    """The download progress has been updated"""

    STARTED = "started"
    """The download has been started"""


@dataclass(frozen=True)
class DownloadBaseEvent:
    filename: str
    filepath: str
    url: str


@dataclass(frozen=True)
class ProgressEventArgs(DownloadBaseEvent):
    downloaded: int
    filesize: int
    percent: int


@dataclass(frozen=True)
class DownloadCompleteEvent(DownloadBaseEvent):
    filesize: int


@dataclass(frozen=True)
class DownloadFailureEvent:
    exception: BaseException | None
    filepath: Optional[str]
    reason: str
    status_code: int | None
    url: str


@dataclass(frozen=True)
class DownloadStartEvent(DownloadBaseEvent):
    filesize: int



from typing import Final


class TerminalColors:
    END_CHAR: Final[str] = '\x1b[00m'
    SUCCESS: Final[str] = '#4CAF50'
    WARNING: Final[str] = '#ff9800'
    ERROR: Final[str] = '#f44336'
    INFO: Final[str] = '#2196F3'
    UNDERLINE: Final[str] = '\x1b[4m'
    BOLD: Final[str] = '\x1b[1m'
    RESET: Final[str] = '\x1b[0m'



from collections.abc import Callable
from typing import TypedDict, NotRequired

from mizue.printer.grid import Alignment, CellRendererArgs


class ColumnSettings(TypedDict):
    alignment: NotRequired[Alignment]
    renderer: NotRequired[Callable[[CellRendererArgs], str]]
    title: str
    width: NotRequired[int]



from typing import Final
from abc import ABC


class BorderCharacterCodes(ABC):
    class Double(ABC):
        TOPLEFT: Final[str] = u'\u2554'  # 0xC9 -> BOX DRAWINGS DOUBLE DOWN AND RIGHT
        TOPRIGHT: Final[str] = u'\u2557'  # 0xBB -> BOX DRAWINGS DOUBLE DOWN AND LEFT
        BOTTOMLEFT: Final[str] = u'\u255a'  # 0xC8 -> BOX DRAWINGS DOUBLE UP AND RIGHT
        BOTTOMRIGHT: Final[str] = u'\u255d'  # 0xBC -> BOX DRAWINGS DOUBLE UP AND LEFT
        TOPMIDDLE: Final[str] = u'\u2566'  # 0xCB -> BOX DRAWINGS DOUBLE DOWN AND HORIZONTAL
        BOTTOMMIDDLE: Final[str] = u'\u2569'  # 0xCA -> BOX DRAWINGS DOUBLE UP AND HORIZONTAL
        LEFTMIDDLE: Final[str] = u'\u2560'  # 0xCC -> BOX DRAWINGS DOUBLE VERTICAL AND RIGHT
        RIGHTMIDDLE: Final[str] = u'\u2563'  # 0xB9 -> BOX DRAWINGS DOUBLE VERTICAL AND LEFT
        MIDDLEMIDDLE: Final[str] = u'\u256c'  # 0xCE -> BOX DRAWINGS DOUBLE VERTICAL AND HORIZONTAL
        HORIZONTAL: Final[str] = u'\u2550'  # 0xCD -> BOX DRAWINGS DOUBLE HORIZONTAL
        VERTICAL: Final[str] = u'\u2551'  # 0xBA -> BOX DRAWINGS DOUBLE VERTICAL

    class Single(ABC):
        TOPLEFT: Final[str] = u'\u250c'  # 0xDA -> BOX DRAWINGS LIGHT DOWN AND RIGHT
        TOPRIGHT: Final[str] = u'\u2510'  # 0xBF -> BOX DRAWINGS LIGHT DOWN AND LEFT
        BOTTOMLEFT: Final[str] = u'\u2514'  # 0xC0 -> BOX DRAWINGS LIGHT UP AND RIGHT
        BOTTOMRIGHT: Final[str] = u'\u2518'  # 0xD9 -> BOX DRAWINGS LIGHT UP AND LEFT
        TOPMIDDLE: Final[str] = u'\u252c'  # 0xC2 -> BOX DRAWINGS LIGHT DOWN AND HORIZONTAL
        BOTTOMMIDDLE: Final[str] = u'\u2534'  # 0xC1 -> BOX DRAWINGS LIGHT UP AND HORIZONTAL
        LEFTMIDDLE: Final[str]= u'\u251c'  # 0xC3 -> BOX DRAWINGS LIGHT VERTICAL AND RIGHT
        RIGHTMIDDLE: Final[str] = u'\u2524'  # 0xB4 -> BOX DRAWINGS LIGHT VERTICAL AND LEFT
        MIDDLEMIDDLE: Final[str] = u'\u253c'  # 0xC5 -> BOX DRAWINGS LIGHT VERTICAL AND HORIZONTAL
        HORIZONTAL: Final[str] = u'\u2500'  # 0xC4 -> BOX DRAWINGS LIGHT HORIZONTAL
        VERTICAL: Final[str] = u'\u2502'  # 0xB3 -> BOX DRAWINGS LIGHT VERTICAL

    class Basic(ABC):
        TOPLEFT: Final[str] = "+"
        TOPRIGHT: Final[str] = "+"
        BOTTOMLEFT: Final[str] = "+"
        BOTTOMRIGHT: Final[str] = "+"
        TOPMIDDLE: Final[str] = "+"
        BOTTOMMIDDLE: Final[str] = "+"
        LEFTMIDDLE: Final[str] = "+"
        RIGHTMIDDLE: Final[str] = "+"
        MIDDLEMIDDLE: Final[str] = "+"
        HORIZONTAL: Final[str] = "-"
        VERTICAL: Final[str] = "|"

    class Empty(ABC):
        TOPLEFT: Final[str] = ""
        TOPRIGHT: Final[str] = ""
        BOTTOMLEFT: Final[str] = ""
        BOTTOMRIGHT: Final[str] = ""
        TOPMIDDLE: Final[str] = ""
        BOTTOMMIDDLE: Final[str] = ""
        LEFTMIDDLE: Final[str] = ""
        RIGHTMIDDLE: Final[str] = ""
        MIDDLEMIDDLE: Final[str] = ""
        HORIZONTAL: Final[str] = ""
        VERTICAL: Final[str] = ""



from .alignment import Alignment
from .border_character_codes import BorderCharacterCodes
from .border_style import BorderStyle
from .cell_renderer_args import CellRendererArgs
from .column_settings import ColumnSettings
# from .column import Column
from .row_border_position import RowBorderPosition
from .grid import Grid

__all__ = [
    "Alignment",
    "BorderStyle",
    "CellRendererArgs",
    "ColumnSettings",
    "Grid",
]



from mizue.printer.grid import ColumnSettings, Alignment


class Column:
    def __init__(self, settings: ColumnSettings):
        self.alignment = settings["alignment"] if "alignment" in settings else Alignment.LEFT
        self.index: int = 0
        self.renderer = settings["renderer"] if "renderer" in settings else None
        self.title = settings["title"] if "title" in settings else ""
        self.width = settings["width"] if "width" in settings else None
        if self.width is not None and self.width <= 0:
            raise ValueError("The column width must be greater than zero")
